@@ -38,12 +38,15 @@ module Database.MySQL.Base
     , executeMany
     , execute_
     , query_
+    , queryVector_
     , query
+    , queryVector
       -- * prepared query statement
     , prepareStmt
     , prepareStmtDetail
     , executeStmt
     , queryStmt
+    , queryStmtVector
     , closeStmt
     , resetStmt
       -- * helpers
@@ -82,6 +85,7 @@ import           Database.MySQL.Protocol.Packet
 import           Database.MySQL.Query
 import           System.IO.Streams                  (InputStream, OutputStream)
 import qualified System.IO.Streams                  as Stream
+import qualified Data.Vector                        as V
 
 --------------------------------------------------------------------------------
 
@@ -123,6 +127,13 @@ execute_ conn (Query qry) = command conn (COM_QUERY qry)
 query :: MySQLConn -> Query -> [MySQLValue] -> IO ([ColumnDef], InputStream [MySQLValue])
 query conn qry params = query_ conn (renderParams qry params)
 
+-- | 'V.Vector' version of 'query'.
+--
+-- @since 0.5.1.0
+--
+queryVector :: MySQLConn -> Query -> [MySQLValue] -> IO (V.Vector ColumnDef, InputStream (V.Vector MySQLValue))
+queryVector conn qry params = queryVector_ conn (renderParams qry params)
+
 -- | Execute a MySQL query which return a result-set.
 --
 query_ :: MySQLConn -> Query -> IO ([ColumnDef], InputStream [MySQLValue])
@@ -144,6 +155,29 @@ query_ conn@(MySQLConn is os _ consumed) (Query qry) = do
                 | otherwise -> Just <$> getFromPacket (getTextRow fields) q
         return (fields, rows)
 
+-- | 'V.Vector' version of 'query_'.
+--
+-- @since 0.5.1.0
+--
+queryVector_ :: MySQLConn -> Query -> IO (V.Vector ColumnDef, InputStream (V.Vector MySQLValue))
+queryVector_ conn@(MySQLConn is os _ consumed) (Query qry) = do
+    guardUnconsumed conn
+    writeCommand (COM_QUERY qry) os
+    p <- readPacket is
+    if isERR p
+    then decodeFromPacket p >>= throwIO . ERRException
+    else do
+        len <- getFromPacket getLenEncInt p
+        fields <- V.replicateM len $ (decodeFromPacket <=< readPacket) is
+        _ <- readPacket is -- eof packet, we don't verify this though
+        writeIORef consumed False
+        rows <- Stream.makeInputStream $ do
+            q <- readPacket is
+            if  | isEOF q  -> writeIORef consumed True >> return Nothing
+                | isERR q  -> decodeFromPacket q >>= throwIO . ERRException
+                | otherwise -> Just <$> getFromPacket (getTextRowVector fields) q
+        return (fields, rows)
+
 -- | Ask MySQL to prepare a query statement.
 --
 prepareStmt :: MySQLConn -> Query -> IO StmtID
@@ -220,7 +254,31 @@ queryStmt conn@(MySQLConn is os _ consumed) stid params = do
         writeIORef consumed False
         rows <- Stream.makeInputStream $ do
             q <- readPacket is
-            if  | isOK  q  -> Just <$> getFromPacket (getBinaryRow fields len) q -- fast path for decode rows
+            if  | isOK  q  -> Just <$> getFromPacket (getBinaryRow fields len) q
+                | isEOF q  -> writeIORef consumed True >> return Nothing
+                | isERR q  -> decodeFromPacket q >>= throwIO . ERRException
+                | otherwise -> throwIO (UnexpectedPacket q)
+        return (fields, rows)
+
+-- | 'V.Vector' version of 'queryStmt'
+--
+-- @since 0.5.1.0
+--
+queryStmtVector :: MySQLConn -> StmtID -> [MySQLValue] -> IO (V.Vector ColumnDef, InputStream (V.Vector MySQLValue))
+queryStmtVector conn@(MySQLConn is os _ consumed) stid params = do
+    guardUnconsumed conn
+    writeCommand (COM_STMT_EXECUTE stid params) os
+    p <- readPacket is
+    if isERR p
+    then decodeFromPacket p >>= throwIO . ERRException
+    else do
+        len <- getFromPacket getLenEncInt p
+        fields <- V.replicateM len $ (decodeFromPacket <=< readPacket) is
+        _ <- readPacket is -- eof packet, we don't verify this though
+        writeIORef consumed False
+        rows <- Stream.makeInputStream $ do
+            q <- readPacket is
+            if  | isOK  q  -> Just <$> getFromPacket (getBinaryRowVector fields len) q
                 | isEOF q  -> writeIORef consumed True >> return Nothing
                 | isERR q  -> decodeFromPacket q >>= throwIO . ERRException
                 | otherwise -> throwIO (UnexpectedPacket q)
